# NoccyLabs\LuaRuntime\Luar  







## Methods

| Name | Description |
|------|-------------|
|[__construct](#luar__construct)||
|[addCoreLibrary](#luaraddcorelibrary)|Adds a "core" library.|
|[addLibrary](#luaraddlibrary)|Adds a library to the interpreter.|
|[assign](#luarassign)|Injects a variable or function into the global scope.|
|[call](#luarcall)|Calls a function in the global scope.|
|[callLuarClosure](#luarcallluarclosure)|Calls a closure generated by Luar.|
|[eval](#luareval)|Evaluates a Lua program.|
|[getGlobals](#luargetglobals)|Returns all the variables in the global scope as an array.|
|[packLuarObject](#luarpackluarobject)|Converts any PHP object to a `LuarObject`|
|[unpackLuarObject](#luarunpackluarobject)|Converts a `LuarObject` to a PHP object|




### Luar::__construct  

**Description**

```php
 __construct (void)
```

 

 

**Parameters**

`This function has no parameters.`

**Return Values**

`void`


<hr />


### Luar::addCoreLibrary  

**Description**

```php
public addCoreLibrary (\Library $library)
```

Adds a "core" library. 

Core library functions get added directly to the root scope and no meta-methods are expected.  
  
```lua  
-- non-core library function example: `string.lower()`  
string.lower('HeLlO, wORLd!')  
  
-- core library function example: `type()`, `print()`  
local var = 'hello world'  
print(var)  
type(var)  
``` 

**Parameters**

* `(\Library) $library`

**Return Values**

`void`


<hr />


### Luar::addLibrary  

**Description**

```php
public addLibrary (\Library $library)
```

Adds a library to the interpreter. 

The functions will be added to a global table. The name of the table will match the name of the library. 

**Parameters**

* `(\Library) $library`

**Return Values**

`void`




<hr />


### Luar::assign  

**Description**

```php
public assign (string $name, mixed $value)
```

Injects a variable or function into the global scope. 

 

**Parameters**

* `(string) $name`
: - Name of the variable  
* `(mixed) $value`
: - variable or function to inject  

**Return Values**

`void`




<hr />


### Luar::call  

**Description**

```php
public call (string $name, array $args)
```

Calls a function in the global scope. 

 

**Parameters**

* `(string) $name`
: - name of the function  
* `(array) $args`
: - arguments to be passed to the function  

**Return Values**

`mixed`




**Throws Exceptions**


`\LuarException`
> - thrown if the name does not resolve to a global function

<hr />


### Luar::callLuarClosure  

**Description**

```php
public callLuarClosure (callable $closure, array $args)
```

Calls a closure generated by Luar. 

Might be useful, for example, if you define the function `myCustomFunction` which takes an argument a function, which you need to call inside the function handler.  
```lua  
myCustomFunction(function (in)  
  local out  
  -- do something...  
  return out  
end)  
``` 

**Parameters**

* `(callable) $closure`
* `(array) $args`

**Return Values**

`mixed`




<hr />


### Luar::eval  

**Description**

```php
public eval (string $program)
```

Evaluates a Lua program. 

 

**Parameters**

* `(string) $program`

**Return Values**

`mixed`




<hr />


### Luar::getGlobals  

**Description**

```php
public getGlobals (void)
```

Returns all the variables in the global scope as an array. 

The names of the variables will be the keys of the array. 

**Parameters**

`This function has no parameters.`

**Return Values**

`array`




<hr />


### Luar::packLuarObject  

**Description**

```php
public static packLuarObject (mixed $value)
```

Converts any PHP object to a `LuarObject` 

 

**Parameters**

* `(mixed) $value`

**Return Values**

`\LuarObject`




<hr />


### Luar::unpackLuarObject  

**Description**

```php
public static unpackLuarObject (\LuarObject $object)
```

Converts a `LuarObject` to a PHP object 

 

**Parameters**

* `(\LuarObject) $object`

**Return Values**

`mixed`




<hr />

